#if defined(__clang__)
#pragma clang diagnostic ignored "-Wsign-compare"
#elif DCMTK_DIAGNOSTIC_MIN_GCC_VERSION(4,6,0)
#pragma GCC diagnostic ignored "-Wsign-compare"
#elif defined(_MSC_VER)
#pragma warning(disable: 4018)
#endif
